import * as vite from "vite";
import { createSettings, openConfig } from "../config/index.js";
import { info } from "../logger/core.js";
import { createContainer } from "./container.js";
import { createSafeError } from "../errors/index.js";
function createRestartedContainer({
  logging,
  fs,
  resolvedRoot,
  configFlag,
  configFlagPath
}, settings) {
  return createContainer({
    isRestart: true,
    logging,
    settings,
    fs,
    root: resolvedRoot,
    configFlag,
    configFlagPath
  });
}
function shouldRestartContainer({
  settings,
  configFlag,
  configFlagPath,
  resolvedRoot,
  restartInFlight
}, changedFile) {
  if (restartInFlight)
    return false;
  let shouldRestart = false;
  shouldRestart = configFlag ? !!configFlagPath && vite.normalizePath(configFlagPath) === vite.normalizePath(changedFile) : new RegExp(
    `${vite.normalizePath(resolvedRoot)}.*astro.config.((mjs)|(cjs)|(js)|(ts))$`
  ).test(vite.normalizePath(changedFile));
  if (!shouldRestart && settings.watchFiles.length > 0) {
    shouldRestart = settings.watchFiles.some(
      (path) => vite.normalizePath(path) === vite.normalizePath(changedFile)
    );
  }
  return shouldRestart;
}
async function restartContainer({
  container,
  flags,
  logMsg,
  handleConfigError
}) {
  const {
    logging,
    close,
    resolvedRoot,
    settings: existingSettings
  } = container;
  container.restartInFlight = true;
  try {
    const newConfig = await openConfig({
      cwd: resolvedRoot,
      flags,
      cmd: "dev",
      logging,
      isRestart: true,
      fsMod: container.fs
    });
    info(logging, "astro", logMsg + "\n");
    let astroConfig = newConfig.astroConfig;
    const settings = createSettings(astroConfig, resolvedRoot);
    await close();
    return createRestartedContainer(container, settings);
  } catch (e) {
    await handleConfigError(e);
    await close();
    info(logging, "astro", "Continuing with previous valid configuration\n");
    return createRestartedContainer(container, existingSettings);
  }
}
async function createContainerWithAutomaticRestart({
  flags,
  handleConfigError = (_e) => {
  },
  params
}) {
  let container = await createContainer(params);
  let resolveRestart;
  let restartComplete = new Promise((resolve) => {
    resolveRestart = resolve;
  });
  let restart = {
    container,
    restarted() {
      return restartComplete;
    }
  };
  function handleServerRestart(logMsg) {
    return async function(changedFile) {
      if (shouldRestartContainer(container, changedFile)) {
        restart.container = await restartContainer({
          container,
          flags,
          logMsg,
          async handleConfigError(_err) {
            const error = createSafeError(_err);
            await handleConfigError(error);
            container.viteServer.ws.send({
              type: "error",
              err: {
                message: error.message,
                stack: error.stack || ""
              }
            });
          }
        });
        resolveRestart();
        restartComplete = new Promise((resolve) => {
          resolveRestart = resolve;
        });
      }
    };
  }
  const watcher = container.viteServer.watcher;
  watcher.on("change", handleServerRestart("Configuration updated. Restarting..."));
  watcher.on("unlink", handleServerRestart("Configuration removed. Restarting..."));
  watcher.on("add", handleServerRestart("Configuration added. Restarting..."));
  return restart;
}
export {
  createContainerWithAutomaticRestart,
  restartContainer,
  shouldRestartContainer
};
